/******************************************************************************
 * 
 * parselayout.cc                                XMM Science Analysis Subsystem
 * (c) 2000  ESA  -  Written Jorgo Bakker, BSC
 * 
 * Synopsis:    dummylayout taskname
 * Requires:    SAS_PATH & SAS_DIR set such that it can find the parameter
 *              and layout file accompanying the task
 * Description: It scans only the taskname.par, and creates a pretty layout
 *           
 * Exit status: 0 on success, <>0 on error
 * Output:      A human readable dump of the layout as existing in memory.
 * 
 *****************************************************************************/
#include "ParameterInfo.h"
#include "LayoutPrinter.h"
#include "LayoutParFile.h"
#include <param.h>
#include <errstr.h>
#include <SasFileLocator.h>

#include <iostream>
#include <string>

int main(int argc, char** argv)
{
  
  errHandler.name("defaultlayout");

  if (argc != 2) 
    errstr << "Missing task-name" 
	   << "Usage: defaultlayout taskname"
	   << Sas::error("MissingArgument");


  // read parameters:
  openParameters(argv[1]);
    
  // create a dummy layout:
  Layout* layout = 0;
  LayoutParFile creator(gParameters);
  if (!creator.create(layout)) 
    errstr << "Could dream-up a proper layout for file("<<argv[1]<<")."
	   << Sas::error("LayoutCreator");

  // print the layout file
  LayoutBuilder* printer = new LayoutPrinter(&cout);
  layout->build(printer);

  delete printer;
  delete layout;

  return 0;
}
