/****************************************************************************
 *
 *  testCistream.cc
 *
 *  Test program for Cistream, Cifstream & Cstringstream classes
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: testCistream.cc,v 1.1 2002/08/29 15:16:14 jbakker Exp $
 *
 *****************************************************************************/

#include "Cistream.h"

#include <iostream>
#include <fstream>
#include <sstream>
#include <string>

#include <cassert>

static void Status(std::istream& is)
{
  std::cout << "is.status = " 
	    << is.good() << is.bad() << is.fail() << is.eof()
	    << ", Available=" << is.rdbuf()->in_avail()
	    << std::endl;
}

void handleNormalReadToEof(std::istream& is)
{
  Status(is);
  assert(is.good());
  assert(!is.bad());
  assert(!is.fail());
  assert(!is.eof());
  
  string n;
  while (is >> n)
    {
      Status(is);
      assert(is.good());
      assert(!is.bad());
      assert(!is.fail());
      assert(!is.eof());
    }
  Status(is);
  
  assert(!is.good());
  assert(!is.bad());
  assert(is.fail());
  assert(is.eof());
  assert(!is);
  //Not gcc 6.1 compliant
  //assert((void*)is == 0);
  
  cout << endl;
}
void handleReadWithError(std::istream& is)
{
  Status(is);
  assert(is.good());
  assert(!is.bad());
  assert(!is.fail());
  assert(!is.eof());
  
  int n;
  while (is >> n)
    {
      Status(is);
      assert(is.good());
      assert(!is.bad());
      assert(!is.fail());
      assert(!is.eof());
    }
  Status(is);
  
  assert(!is.good());
  assert(!is.bad());
  assert(is.fail());
  assert(!is.eof());
  assert(!is);
  //Not gcc 6.1 compliant
  //assert((void*)is == 0);
  
  cout << endl;
}

void handleNonExistantFile(std::istream& is)
{
  Status(is);
  /* GCC-2.95
  int flags=is.rdbuf()->_flags;
  cout << "F=" << hex << flags
       << ",BAD =" << (flags&_IO_BAD_SEEN)
       << ",FAIL=" << (flags&_IO_ERR_SEEN)
       << ",EOF =" << (flags&_IO_EOF_SEEN)
       << ",!REA=" << (flags&_IO_NO_READS)
       << ",!WRI=" << (flags&_IO_NO_WRITES) << endl;
  */

  assert(!is.good());
  //assert(is.bad());
  assert(is.fail());
  assert(!is.eof());
  
  cout << endl;
}

int main()
{
  /*** normal read to EOF ***/
  {
    cout << "normal read to EOF [istringstream]" << endl;
    istringstream is("12\n34\n" "45\n" "xyz\n" "ab cd\n");
    handleNormalReadToEof(is);
  }
  {
    cout << "normal read to EOF [Cistream(istringstream.rdbuf())]" << endl;
    istringstream iss("12\n34\n" "45\n" "xyz\n" "ab cd\n");
    Cistream is(iss.rdbuf());
    handleNormalReadToEof(is);
  }
  {
    cout << "normal read to EOF [ifstream]" << endl;
    ifstream is("testCistream.dat");
    handleNormalReadToEof(is);
  }
  {
    cout << "normal read to EOF [Cistream(ifstream.rdbuf())]" << endl;
    ifstream iss("testCistream.dat");
    Cistream is(iss.rdbuf());
    handleNormalReadToEof(is);
  }


  /*** read with error ***/
  {
    cout << "read with error [istringstream]" << endl;
    istringstream is("12\n34\n" "45\n" "xyz\n" "ab cd\n");
    handleReadWithError(is);
  }
  {
    cout << "read with error [Cistream(istringstream.rdbuf())]" << endl;
    istringstream iss("12\n34\n" "45\n" "xyz\n" "ab cd\n");
    Cistream is(iss.rdbuf());
    handleReadWithError(is);
  }
  {
    cout << "read with error [ifstream]" << endl;
    ifstream is("testCistream.dat");
    handleReadWithError(is);
  }
  {
    cout << "read with error [Cistream(ifstream.rdbuf())]" << endl;
    ifstream iss("testCistream.dat");
    Cistream is(iss.rdbuf());
    handleReadWithError(is);
  }

  /*** existant file ***/
  {
    cout << "non-existant file [ifstream]" << endl;
    ifstream is("nonexistant.dat");
    handleNonExistantFile(is);
  }
  {
    cout << "non-existant file [Cistream(ifstream.rdbuf())]" << endl;
    ifstream iss("nonexistant.dat");
    Cistream is(iss.rdbuf());
    handleNonExistantFile(is);
  }


  /*** non-existant file ***/
  {
    cout << "non-existant file [ifstream]" << endl;
    ifstream is("nonexistant.dat");
    handleNonExistantFile(is);
  }
  {
    cout << "non-existant file [Cistream(ifstream.rdbuf())]" << endl;
    ifstream iss("nonexistant.dat");
    Cistream is(iss.rdbuf());
    handleNonExistantFile(is);
  }

  return 0;
}
